﻿-- **********************************************************************
-- GnomTEC Gnomcorder Positioner
-- Version: 0.3
-- Author: Lugus Sprengfix
-- Copyright 2011-2012 by GnomTEC
-- http://www.gnomtec.de/
-- **********************************************************************
-- load localization first.
local L = LibStub("AceLocale-3.0"):GetLocale("GnomTEC_Gnomcorder_Positioner")

-- ----------------------------------------------------------------------
-- Addon global Constants (local)
-- ----------------------------------------------------------------------

-- Prefix for addon communication
-- local CONST_ADDONMESSAGE_PREFIX = "GNOMTEC_POSITIONER"

-- ----------------------------------------------------------------------
-- Addon global variables (local)
-- ----------------------------------------------------------------------

-- Main options menue with general addon information
local optionsMain = {
	name = "GnomTEC Gnomcorder Positioner",
	type = "group",
	args = {
		descriptionTitle = {
			order = 1,
			type = "description",
			name = "Positionierungsmodul für GnomTEC Gnomcorder.\n\n",
		},
		descriptionAbout = {
			name = "About",
			type = "group",
			guiInline = true,
			order = 2,
			args = {
				descriptionVersion = {
				order = 1,
				type = "description",			
				name = "|cffffd700".."Version"..": ".._G["GREEN_FONT_COLOR_CODE"]..GetAddOnMetadata("GnomTEC_Gnomcorder_Positioner", "Version"),
				},
				descriptionAuthor = {
					order = 2,
					type = "description",
					name = "|cffffd700".."Autor"..": ".."|cffff8c00".."Lugus Sprengfix",
				},
				descriptionEmail = {
					order = 3,
					type = "description",
					name = "|cffffd700".."Email"..": ".._G["HIGHLIGHT_FONT_COLOR_CODE"].."info@gnomtec.de",
				},
				descriptionWebsite = {
					order = 4,
					type = "description",
					name = "|cffffd700".."Website"..": ".._G["HIGHLIGHT_FONT_COLOR_CODE"].."http://www.gnomtec.de/",
				},
				descriptionLicense = {
					order = 5,
					type = "description",
					name = "|cffffd700".."Copyright"..": ".._G["HIGHLIGHT_FONT_COLOR_CODE"].."(c)2011 by GnomTEC",
				},
			}
		},
		descriptionLogo = {
			order = 5,
			type = "description",
			name = "",
			image = "Interface\\AddOns\\GnomTEC_Gnomcorder\\Textures\\GnomTEC-Logo",
			imageCoords = {0.0,1.0,0.0,1.0},
			imageWidth = 512,
			imageHeight = 128,
		},
	}
}

-- Last time when a timer event was triggerd
local lastTimerEvent = GetTime()
local lockedFacing = nil
local lockedZone = nil
local lockedPosX = nil
local lockedPosY = nil


-- ----------------------------------------------------------------------
-- Startup initialization
-- ----------------------------------------------------------------------

GnomTEC_Gnomcorder_Positioner = LibStub("AceAddon-3.0"):NewAddon("GnomTEC_Gnomcorder_Positioner", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0", "AceComm-3.0")
local Tourist = LibStub("LibTourist-3.0") 
local BZ = LibStub("LibBabble-Zone-3.0"):GetLookupTable()


LibStub("AceConfig-3.0"):RegisterOptionsTable("GnomTEC Gnomcorder Positioner Main", optionsMain)
LibStub("AceConfigDialog-3.0"):AddToBlizOptions("GnomTEC Gnomcorder Positioner Main", "GnomTEC Gnomcorder Positioner");

-- ----------------------------------------------------------------------
-- Local functions
-- ----------------------------------------------------------------------

-- function which returns also nil for empty strings
local function emptynil( x ) return x ~= "" and x or nil end

-- example for position handling
function GnomTEC_Gnomcorder_Positioner:SendPosition()
	local player = UnitName("player")
	local posX, posY = GetPlayerMapPosition("player");
	local map = GetMapInfo();
	local zone = nil;
	
	if not map then
		if GetCurrentMapZone() == 0 then
			if GetCurrentMapContinent() == 0 then
				zone = "Azeroth";
			elseif GetCurrentMapContinent() == -1 then
				zone = "Cosmic map";
			end
		end
	else
		zone = Tourist:GetEnglishZoneFromTexture(map);
	end
		
	if (posX ~= 0 and posY ~= 0 and zone) then

		GnomTEC_Gnomcorder_Positioner:ShowPosition(player, zone, posX, posY)	
		if IsInGuild() then
			GnomTEC_Gnomcorder_Positioner:SendCommMessage(CONST_ADDONMESSAGE_PREFIX, zone.."|"..posX.."|"..posY, "GUILD", nil, "BULK")
		end
	end
end


-- ----------------------------------------------------------------------
-- Frame event handler and functions
-- ----------------------------------------------------------------------

-- handler for timer events 
function GnomTEC_Gnomcorder_Positioner:TimerEvent()
	local t = GetTime()
	local facing = GetPlayerFacing()
	
	GNOMTEC_GNOMCORDER_POSITIONER_FRAME_COMPASS:SetRotation(-facing)	
	if (lockedFacing == nil) then
		GNOMTEC_GNOMCORDER_POSITIONER_FRAME_COMPASSMARKER:SetRotation(-facing-math.pi)	
		GNOMTEC_GNOMCORDER_POSITIONER_FRAME_COMPASSMARKER:SetPoint("CENTER", 90*math.sin(facing), 90*math.cos(-facing))
		GNOMTEC_GNOMCORDER_POSITIONER_FRAME_TARGET:SetPoint("CENTER", 0, 0)
	else
		local player = UnitName("player")
		local posX, posY = GetPlayerMapPosition("player");
		local map = GetMapInfo();
		local zone = nil;
	
		if not map then
			if GetCurrentMapZone() == 0 then
				if GetCurrentMapContinent() == 0 then
					zone = "Azeroth";
				elseif GetCurrentMapContinent() == -1 then
					zone = "Cosmic map";
				end
			end
		else
			zone = Tourist:GetEnglishZoneFromTexture(map);
		end

		if (posX ~= 0 and posY ~= 0 and zone) then
			if (zone == "Stormwind") then
				-- Stormwind kennt LibTourist so nicht
				zone = "Stormwind City"
			end

			posX, posY = Tourist:TransposeZoneCoordinate(posX, posY, BZ[zone], BZ[lockedZone])
			local distance = Tourist:GetYardDistance(BZ[lockedZone], posX, posY, BZ[lockedZone], lockedPosX, lockedPosY)
			posX = lockedPosX - posX
			posY = lockedPosY - posY

			if (distance == nil) then
				distance = 10
			elseif (distance > 10) then
				distance = 10
			end
			
			if (distance == 0) then
				GNOMTEC_GNOMCORDER_POSITIONER_FRAME_TARGET:SetPoint("CENTER", 0, 0)			
			else		
				local rad = math.acos(posY/math.sqrt(posX*posX + posY*posY))
				
				if (posX < 0) then
					rad = 2*math.pi - rad
				end
				
				rad = rad - facing

				GNOMTEC_GNOMCORDER_POSITIONER_FRAME_TARGET:SetPoint("CENTER", distance * 10 * math.sin(rad), (-distance) * 10 * math.cos(rad))
			end		

			
		end
	end
	lastTimerEvent = t
end

-- lock the actual position and facing
function GnomTEC_Gnomcorder_Positioner:LockActualPosition()
	local player = UnitName("player")
	local posX, posY = GetPlayerMapPosition("player");
	local map = GetMapInfo();
	local zone = nil;
	
	if not map then
		if GetCurrentMapZone() == 0 then
			if GetCurrentMapContinent() == 0 then
				zone = "Azeroth";
			elseif GetCurrentMapContinent() == -1 then
				zone = "Cosmic map";
			end
		end
	else
		zone = Tourist:GetEnglishZoneFromTexture(map);
	end
		
	if (posX ~= 0 and posY ~= 0 and zone) then
		if (zone == "Stormwind") then
			-- Stormwind kennt LibTourist so nicht
			zone = "Stormwind City"
		end

		lockedFacing = GetPlayerFacing()
		lockedZone = zone
		lockedPosX = posX
		lockedPosY = posY
		GNOMTEC_GNOMCORDER_POSITIONER_FRAME_COMPASSMARKER:SetRotation(-lockedFacing-math.pi)
		GNOMTEC_GNOMCORDER_POSITIONER_FRAME_COMPASSMARKER:SetPoint("CENTER", 90*math.sin(lockedFacing), 90*math.cos(-lockedFacing))
		GNOMTEC_GNOMCORDER_POSITIONER_FRAME_TARGET:SetPoint("CENTER", 0, 0)
	end
end


-- ----------------------------------------------------------------------
-- Hook functions
-- ----------------------------------------------------------------------

-- ----------------------------------------------------------------------
-- Event handler
-- ----------------------------------------------------------------------


-- event handler for addon communication messages
function GnomTEC_Gnomcorder_Positioner:OnCommReceived(prefix, message, distribution, sender)
	if (prefix == CONST_ADDONMESSAGE_PREFIX) then
--
	end
end

-- ----------------------------------------------------------------------
-- Addon OnInitialize, OnEnable and OnDisable
-- ----------------------------------------------------------------------

-- function called on initialization of addon
function GnomTEC_Gnomcorder_Positioner:OnInitialize()
 	-- Code that you want to run when the addon is first loaded goes here.
	self.db = LibStub("AceDB-3.0"):New("GnomTEC_Gnomcorder_PositionerDB")

  	GnomTEC_Gnomcorder_Positioner:Print("Willkommen bei GnomTEC_Gnomcorder_Positioner")
  	  	
end

-- function called on enable of addon
function GnomTEC_Gnomcorder_Positioner:OnEnable()
    -- Called when the addon is enabled

	GnomTEC_Gnomcorder_Positioner:Print("GnomTEC_Gnomcorder_Positioner Enabled")

	-- Initialize options which are propably not valid because they are new added in new versions of addon
--	if (not self.db.char.displayedMap) then
--		if (not GnomTEC_Gnomcorder_Positioner_Options["DisplayedMap"]) then
--			self.db.char.displayedMap = "Stormwind City"			
--		else
--			-- get legacy option
--			self.db.char.displayedMap = GnomTEC_Gnomcorder_Positioner_Options["DisplayedMap"]
--		end
--	end
		
	-- set local parameters
	
	-- initialize hooks and events
--	RegisterAddonMessagePrefix(CONST_ADDONMESSAGE_PREFIX);
--	GnomTEC_Gnomcorder_Positioner:RegisterComm(CONST_ADDONMESSAGE_PREFIX)
	
	-- Add module to GnomTEC Gnomcorder
	GnomTEC_Gnomcorder:AddButton("Positioner", "Positionierer", L["L_SHOWPOSITIONER"], GNOMTEC_GNOMCORDER_POSITIONER_FRAME, "Interface\\ICONS\\INV_MISC_IDOL_05", "Interface\\ICONS\\INV_MISC_IDOL_05", false, nil, nil)
end

-- function called on disable of addon
function GnomTEC_Gnomcorder_Positioner:OnDisable()
    -- Called when the addon is disabled
    GnomTEC_Gnomcorder_Positioner:UnregisterAllEvents();
end

-- ----------------------------------------------------------------------
-- External API
-- ----------------------------------------------------------------------
